c=======================================================================
c betafn
c   incomplete Beta function B(x,a,b)
c   over 0 <= x <= 1 and all a and b both positive and negative.
c
c A. J. S. Hamilton
c Andrew.Hamilton@colorado.edu
c http://casa.colorado.edu/~ajsh/
c
c---Start of revision history-------------------------------------------
c
c Originally coded around 1990.
c
c 6 June 2000:
c First published version.
c
c 10 June 2000:
c Revised definition of B(x,a,b) for negative integral a = 0, -1, ...
c so that it satisfies the recurrence relations.
c
c 13 June 2000:
c Separated betafp, to deal accurately with case of tiny 1-x.
c
c---End of revision history---------------------------------------------
c
c=======================================================================
      real*8 function betafn(x,a,b)
      real*8 x,a,b
c
c        externals
      real*8 betafp
c        local (automatic) variables
      real*8 xp
c
c        define xp=1-x
c Note some loss of precision can occur from just writing
c xp=1-x, for small xp; for example, if x=.999999, then
c 1-x may not be precisely .000001, because of roundoff.
c Routine betafp works accurately even for tiny xp.
      xp=1.d0-x
      betafn=betafp(x,xp,a,b)
      return
      end
c
c-----------------------------------------------------------------------
      real*8 function betafp(x,xp,a,b)
      real*8 x,xp,a,b
c
c        externals
      real*8 dflmax,expf,log1p
      real*8 betaff,betafs,betaft
c        data variables
      real*8 aerr,rerr
      real*8 lerr
c        local (automatic) variables
      integer i,na,nb
      real*8 aa,bb,bf,bfc,bfn,bfr,bfs,h,hp
c *
c * Computes incomplete Beta function
c *                x
c *               /  a-1      b-1
c *    B (a,b) == | t    (1-t)    dt
c *     x         /
c *              0
c * over 0 <= x <= 1 , and all a and b both positive and negative.
c *
c * xp = 1-x is provided as an argument to permit accurate computation
c * in cases where xp is tiny.
c *
c * For a <= 0, betafn computes the analytic continuation of the
c * integral expression above, in accordance with the usual convention.
c *
c * The case x = 1 corresponds to the complete Beta function, which is
c *                        Gamma(a) Gamma(b)
c *    B(a,b) == B (a,b) = -----------------
c *               1           Gamma(a+b)
c *
c * For x = 0, the (analytically continued) incomplete Beta function is
c *    B (a,b) = 0      if a > 0
c *     0        -oo    if a <= 0
c *
c * If a is a non-positive integer, i.e. a = -n = 0, -1, -2, ...,
c * then (the analytic continuation of) B_x(a,b) is infinite.
c * Here betafn returns not B_x(-n,b), but rather
c * (the analytic continuation of, if b >= n+1)
c *
c *        x
c *       /  -n-1      b-1          pi    (1-b)_n
c *       | t     (1-t)    dt = --------- -------  -  B   (b,-n)
c *       /                     tan(pi b)    n!        1-x
c *  -Infinity
c *
c * where (1-b)_n = (1-b)(2-b)...(n-b) is a Pochhammer symbol.
c * The advantage of this choice is that the usual recurrence
c * relations remain valid.  Moreover the definition works
c * for all b, including negative (and positive) integral b.
c * This revised incomplete Beta function is infinite if b is a
c * positive integer exceeding n, that is if b = n+1, n+2, ... .
c *
c * The above definition of the incomplete Beta function for negative
c * integral a = -n = 0, -1, -2, ...  is adhered to also for x = 1.
c * Thus for x = 1, betafn returns not the usual complete Beta function
c * (which is infinite for negative integral a), but rather
c *        pi    (1-b)_n
c *    --------- -------
c *    tan(pi b)    n!
c *
c * B_x(a,b) is in general complex for x outside the interval [0,1],
c * and betafn does not deal with this case.
c *
c * Note that:
c *  Infinity
c *     /  a-1      b-1
c *     | t    (t-1)    dt = B   (1-a-b,b)              (x >= 1)
c *     /                     1/x
c *    x
c *
c *      x
c *     /  a-1      b-1
c *     | t    (1+t)    dt = B       (a,1-a-b)          (x >= 0)
c *     /                     x/(1+x)
c *    0
c *
c *  Infinity
c *     /  a-1      b-1
c *     | t    (1+t)    dt = B       (1-a-b,a)          (x >= 0)
c *     /                     1/(1+x)
c *    x
c *
c *      0
c *     /  a-1      b-1           a
c *     | t    (1-t)    dt = - (-)  B       (a,1-a-b)   (x >= 0)
c *     /                            x/(1+x)
c *   -x
c *
c *      x
c *     /  a-1      b-1           b
c *     | t    (1-t)    dt = - (-)  B       (b,1-a-b)   (x >= 1)
c *     /                            (x-1)/x
c *    1
c *
c * Warning: the routine may be slow for large negative a or b.
c *
      data aerr /-1.d0/
      data lerr /-16.d0/
      data rerr /1.d-16/
c
c some fortran compilers do not handle exp(+-infinity) correctly
#ifndef EXP_IS_BAD
      expf(h)=exp(h)
#endif
c
c        check range of x is OK
      if (x.lt.0.d0) goto 430
      if (xp.lt.0.d0) goto 440
      if (abs(x+xp-1.d0).gt.10.d0*rerr) goto 450
c--------case x=0
      if (x.eq.0.d0) then
        bfn=0.d0
        if (a.le.0.d0) then
          bfn=-1.d0/bfn
        endif
        goto 290
      endif
c--------transform B(x,a,b) = bfs + bfc*B(x,aa,bb)
c........transform complete Beta function so aa and bb >= 0.5
      if (xp.eq.0.d0.and..not.(a.eq.anint(a).and.a.le.0.d0)) then
        bfs=0.d0
        call betafc(a,b,aa,bb,bfc)
c        h to split B(1,aa,bb) = B(h,aa,bb) + B(1-h,bb,aa)
        call betafh(aa,bb,h,hp)
c........transform incomplete Beta function so aa [and bb] >= -0.5
      else
c        na is integer such that aa=a+na is in interval (-0.5,0.5]
        na=0.5d0-a
        if (na.lt.0) na=0
        aa=a+na
c        nb is integer such that bb=b+nb is in interval (-0.5,0.5]
        nb=0.5d0-b
        if (nb.lt.0) nb=0
        bb=b+nb
c        h to split B(1,aa,bb) = B(h,aa,bb) + B(1-h,bb,aa)
        call betafh(aa,bb,h,hp)
c Transforming b destabilizes B(x,a,b) for small x,
c at least if a > 0 and b < -a.
c Not transforming b destabilizes B(x,a,b) if a < 0.
        if (a.gt.0..and.
     &    ((h.le.0.5d0.and.x.le.h).or.(h.gt.0.5d0.and.xp.ge.hp))) then
          nb=0
          bb=b
          h=1.d0
          hp=0.d0
        endif
c        transform incomplete Beta function so aa [and bb] >= -0.5
        call betafi(x,xp,a,b,na,nb,bfs,bfc)
      endif
c--------if multiplier is zero or infinite, skip to end of program
      if (bfc.eq.0.d0.and.aa.ne.0.d0) then
        bfn=bfs
        goto 290
      elseif (abs(bfc).gt.dflmax()) then
        bfn=bfc
        goto 290
      elseif (abs(bfs).gt.dflmax()) then
        bfn=bfs
        goto 290
      endif
c--------special case aa=0 (i.e. a=0,-1,-2,...) with a+b=-ve integer
      if (aa.eq.0.d0) then
        if (bfc.eq.0.d0) then
          nb=nint(b)
          if (b.eq.dble(nb).and.nb.ge.1.and.na.ge.nb) then
            bfc=1.d0
c        pi/tan(pi b) (1-b)_n/n!, for a=-n=-1,-2,..., b=n,n+1,...
            bfn=1.d0/b
            if (mod(nb,2).eq.1) bfn=-bfn
            do i=1,na-nb
               bfn=bfn*i/(b+i)
            enddo
          endif
c--------special case aa=0 with x < h:
c        pi/tan(pi bb) + B(x,aa,bb) - B(h,aa,bb) - B(1-h,bb,aa)
        elseif ((h.le.0.5d0.and.x.lt.h).or.(h.gt.0.5d0.and.xp.gt.hp))
     &    then
          bfn=betafs(x,aa,bb,0,lerr)
          bfn=bfn-betafs(h,aa,bb,0,lerr)
          bfn=bfn-betafs(hp,bb,aa,0,lerr)
          bfn=betaft(bb)+bfn
c--------special case aa=0 with x >= h:
c        pi/tan(pi bb) - B(1-x,bb,aa)
        else
c        betafs drops constant 1/bb from B(1-x,bb,aa)
          bfn=-betafs(xp,bb,aa,0,lerr)
c        betaft drops constant 1/bb from pi/tan(pi bb)
          bfn=betaft(bb)+bfn
        endif
c--------case aa!=0 and x <= h: B(x,aa,bb)
      else
        if ((h.le.0.5d0.and.x.le.h).or.(h.gt.0.5d0.and.xp.ge.hp)) then
c        use continued fraction
          bf=betaff(x,xp,aa,bb,rerr,aerr)
          if (x.le.0.5d0) then
            bfr=expf(aa*log(x)+bb*log1p(-x))
          elseif (x.gt.0.5d0) then
            bfr=expf(aa*log1p(-xp)+bb*log(xp))
          endif
          bfn=bfr*bf
c--------case aa !=0 and x > h:
c        B(x,aa,bb) = B(1,aa,bb) - B(1-x,bb,aa)
c                   = B(h,aa,bb) + B(1-h,bb,aa) - B(1-x,bb,aa)
        else
c........B(1,aa,bb) = B(h,aa,bb) + B(1-h,bb,aa)
c        algorithm is the same for both h <= 0.5 and h > 0.5,
c        but it's inverted symmetrically for precision and efficiency
          if (h.le.0.5d0) then
            bfr=expf(aa*log(h)+bb*log1p(-h))
            bfn=bfr*betaff(h,hp,aa,bb,rerr,aerr)
c        case bb >= 0.5
            if (bb.ge.0.5d0) then
c        use continued fraction
              bfn=bfn+bfr*betaff(hp,h,bb,aa,rerr,rerr*abs(bfn/bfr))
c        case bb < 0.5 (note implies xp != 0, through betafc)
            else
c        use series, to avoid near cancellation of large terms
c        note hp = bb+1/(aa+bb+1) < .75 for bb < .5
              bfn=bfn+betafs(hp,bb,aa,0,lerr)
            endif
c        same deal, but for h > 0.5
          else
            bfr=expf(aa*log1p(-hp)+bb*log(hp))
            if (bb.ge.0.5d0) then
              bfn=bfr*betaff(hp,h,bb,aa,rerr,aerr)
            elseif (bb.lt.0.5d0) then
              bfn=betafs(hp,bb,aa,0,lerr)
            endif
            bfn=bfn+bfr*betaff(h,hp,aa,bb,rerr,rerr*abs(bfn/bfr))
          endif
c........subtract off B(1-x,bb,aa)
          if (xp.gt.0.d0) then
            if (bb.ge.0.5d0) then
              if (x.le.0.5d0) then
                bfr=expf(aa*log(x)+bb*log1p(-x))
              else
                bfr=expf(aa*log1p(-xp)+bb*log(xp))
              endif
              bfn=bfn-bfr*betaff(xp,x,bb,aa,rerr,aerr)
            else
              bfn=bfn-betafs(xp,bb,aa,0,lerr)
            endif
          endif
        endif
      endif
c--------B(x,a,b) = bfs + bfc*B(x,aa,bb)
      bfn=bfs+bfn*bfc
c        skip to here in simple cases
  290 continue
c     if (x.eq.1.d0) then
c       if (b.eq.1.d0) then
c         print *,bfn*a-1.d0
c       elseif (b.eq.2.d0) then
c         print *,bfn*a*(a+1.d0)-1.d0
c       elseif (b.eq.3.d0) then
c         print *,bfn*a*(a+1.d0)*(a/2.d0+1.d0)-1.d0
c       elseif (b.eq.4.d0) then
c         print *,bfn*a*(a+1.d0)*(a/2.d0+1.d0)*(a/3.d0+1.d0)-1.d0
c       endif
c     endif
      betafp=bfn
      return
c
c--------error returns
  430 print *,'from betafp: argument x =',x,' should be >= 0.'
      betafp=x
      return
  440 print *,'from betafp: argument 1-x =',xp,' should be >= 0.'
      betafp=x
      return
  450 print *,'from betafp: arguments x + 1-x =',x+xp,' should = 1.'
      betafp=x
      return
c
      end
c
c-----------------------------------------------------------------------
      subroutine betafc(a,b,aa,bb,bfc)
      real*8 a,b,aa,bb,bfc
c
c        externals
      real*8 dflmax
c        local (automatic) variables
      integer i,na,nb
      real*8 da,db
c *
c * Transform complete Beta function by
c *    B(1,a,b) = bfc*B(1,aa,bb)
c * with aa and bb >= 0.5 .
c * Used by real*8 function betafp.
c *
c * The purpose is to get around the problems of
c * (a) zero multiplying factors in numerator and denominator;
c * (b) loss of accuracy of continued fraction for B(x,a,b) when a < 0,
c *     or B(xp,b,a) when b < 0.
c *
c * Strategy: order factors so they tend to cancel each other,
c * doing the most extreme (not close to 1) factors first.
c *
c Input:  a, b = parameters of complete Beta function
c Output: aa, bb = >= 0.5 parameters of another complete Beta function
c         bfc = multiplication factor
c
      bfc=1.d0
c--------case where both a and b are >= 0.5 : do nothing
      if (a.ge.0.5d0.and.b.ge.0.5d0) then
        aa=a
        bb=b
      elseif (a.lt.0.5d0.or.b.lt.0.5d0) then
        aa=min(a,b)
        bb=max(a,b)
c        na is integer such that aa+na is in interval [-0.5,0.5)
        na=0.5d0-aa
        if (aa+na.ge.0.5d0) na=na-1
        da=aa+na
c--------case where one of a or b is < 0.5, and the other is >= 0.5
        if (bb.ge.0.5d0) then
c        test equivalent to na.lt.nb, but avoids overflow for large bb
          if (dble(na).le.bb-0.5d0) then
c        following factors are all large and negative (i.e. < -1)
            do 120 i=0,na
              bfc=bfc*(aa+bb+i)/(da-i)
              if (abs(bfc).gt.dflmax()) goto 320
  120       continue
          elseif (dble(na).gt.bb-0.5d0) then
c        nb is integer such that bb-nb is in interval [-0.5,0.5)
            nb=bb+0.5d0
            db=bb-nb
c        following factor liable to be the most extreme
c        beware possibility that da=da+db=0
            if (da+db.ne.da) bfc=(da+db)/da
            if (bfc.eq.0.d0) goto 310
            if (abs(bfc).gt.dflmax()) goto 320
c        following factors are small and negative (.i.e. in (-1,0))
            do 140 i=1,nb
              bfc=bfc*(da+db+i)/(aa+nb-i)
              if (bfc.eq.0.d0) goto 310
  140       continue
c        following factors are all of order unity
            if (da+db-1.d0.ne.da-1.d0) then
              do 150 i=1,na-nb
                bfc=bfc*(da+db-i)/(da-i)
  150         continue
            endif
          endif
          if (a.le.b) then
            aa=da+1.d0
            bb=b
          elseif (a.gt.b) then
            aa=a
            bb=da+1.d0
          endif
c--------case where both a and b are < 0.5
        elseif (bb.lt.1.d0) then
c        nb is integer such that bb+nb is in interval [-0.5,0.5)
          nb=0.5d0-bb
          if (bb+nb.ge.0.5d0) nb=nb-1
          db=bb+nb
          if (da.eq.0.d0) then
            bfc=bfc/da
            goto 320
          elseif (db.eq.0.d0) then
            bfc=bfc/db
            goto 320
          endif
          bfc=bfc*(da+bb)/da
          bfc=bfc*(da+db+1.d0)/db
          if (bfc.eq.0.d0) goto 310
          if (abs(bfc).gt.dflmax()) goto 320
c        following factors are large and positive (i.e. > 1)
          do 220 i=1,na
            bfc=bfc*(da+bb-i)/(da-i)
            if (abs(bfc).gt.dflmax()) goto 320
  220     continue
c        following factors are of order unity
          do 230 i=1,nb
            bfc=bfc*(da+db+1-i)/(db-i)
  230     continue
          if (a.le.b) then
            aa=da+1.d0
            bb=db+1.d0
          elseif (a.gt.b) then
            aa=db+1.d0
            bb=da+1.d0
          endif
        endif
      endif
c     print *,'betafc: bfc=',bfc
      return
c        exit here if factor is zero
  310 return
c        exit here if factor is infinite
  320 return
      end
c
c-----------------------------------------------------------------------
      subroutine betafi(x,xp,a,b,na,nb,bfs,bfc)
      integer na,nb
      real*8 x,xp,a,b,bfs,bfc
c
c        externals
      real*8 dflmax,expf,log1p
c        local (automatic) variables
      integer ia,ib,n,nab
      real*8 aa,aam,bb,bbm
c *
c * Transform Incomplete Beta function by
c *    B(x,a,b) = bfs + bfc*B(x,a+na,b+nb)
c * through successive applications of
c *               a     b
c *              x (1-x)    (a+b)
c *    B (a,b) = -------- + ----- B (a+1,b)
c *     x           a         a    x
c * and
c *                 a     b
c *                x (1-x)    (a+b)
c *    B (a,b) = - -------- + ----- B (a,b+1)
c *     x             b         b    x
c *
c * Used by real*8 function betafp.
c *
c * The purpose is to get around the problems of
c * (a) singularity at a = -1, -2, ...;
c * (b) loss of accuracy of continued fraction for B(x,a,b) when a < 0,
c *     or B(xp,b,a) when b < 0.
c *
c Input:  x, xp=1-x = argument of Incomplete Beta function
c         a, b are parameters of Incomplete Beta function
c         na = increase a to a+na; na should be >= 0
c         nb = increase b to b+nb; nb should be >= 0
c Output: bfs = additive term
c         bfc = multiplication factor
c
#ifndef EXP_IS_BAD
      real*8 y
      expf(y)=exp(y)
#endif
c
      bfs=0.d0
      bfc=1.d0
      if (x.eq.0.d0) goto 200
      if (na.le.0.and.nb.le.0) goto 200
      ia=na
      ib=nb
c        worry about case where a+b+nab=0 for some nab
      nab=nint(-a-b)
      if (a+b+nab.eq.0.d0.and.nab.ge.0) then
c        can skip 1st n terms
        n=max(na+nb-nab-1,0)
c        preferaby skip transforming b
        if (n.le.ib) then
          ib=ib-n
          n=0
        else
          n=n-ib
          ib=0
          ia=ia-n
        endif
      endif
      if (ia.le.0.and.ib.le.0) goto 200
c        starting values for transformation
      aa=a+ia
      ia=ia-1
      if (ia.ge.0) then
        aam=a+ia
      else
        aam=1.d0
      endif
      bb=b+ib
      ib=ib-1
      if (ib.ge.0) then
        bbm=b+ib
      else
        bbm=1.d0
      endif
c        transformation loop
  120 if (ia.lt.0.and.ib.lt.0) goto 140
c        order transformations so smallest factors appear most often,
c        i.e. largest factors are done first
c        aam and bbm are negative until done, whereat they are set to 1
        if (x/aam.le.xp/bbm) then
          aa=aam
c           print *,'a',ia,'bfs,bfc facs:',bfs*x*(aa+bb),(aa+bb)/aa
          bfs=(bfs*x*(aa+bb)+1.d0)/aa
          bfc=bfc*(aa+bb)/aa
          ia=ia-1
          if (ia.ge.0) then
            aam=a+ia
          else
            aam=1.d0
          endif
        elseif (x/aam.gt.xp/bbm) then
          bb=bbm
c           print *,'b',ib,'bfs,bfc facs:',-bfs*xp*(aa+bb),(aa+bb)/bb
          bfs=(bfs*xp*(aa+bb)-1.d0)/bb
          bfc=bfc*(aa+bb)/bb
          ib=ib-1
          if (ib.ge.0) then
            bbm=b+ib
          else
            bbm=1.d0
          endif
        endif
        if (abs(bfs).ge.dflmax()) goto 320
        if (abs(bfc).ge.dflmax()) goto 320
        goto 120
c        transformation loop finished
  140 continue
c        multiply bfs by x^a*(1-x)^b
      if (x.le.0.5d0) then
        bfs=bfs*expf(a*log(x)+b*log1p(-x))
      elseif (x.gt.0.5d0) then
        bfs=bfs*expf(a*log1p(-xp)+b*log(xp))
      endif
      if (abs(bfs).ge.dflmax()) goto 320
  200 continue
      return
c
c--------error returns
c        exit here if bfs or bfc are infinite
  320 return
      end
c
c-----------------------------------------------------------------------
      real*8 function betaff(x,xp,a,b,rerr,aerr)
      real*8 x,xp,a,b,rerr,aerr
c
c        externals
      real*8 dflmax
c        local (automatic) variables
      integer i,imax
      real*8 bde,bdo,bne,bno,dn,dd,err,r
c     real*8 be,ee
c *
c * Repeated fraction factor used by real*8 function betafp
c * to compute Incomplete Beta function
c *                a     b
c *    B(x,a,b) = x (1-x) * betaff
c *
c Input:  x, xp=1-x = argument of Incomplete Beta function
c         a, b = parameters of Incomplete Beta function
c         rerr = desired relative accuracy of factor (e.g. 1.d-16)
c         aerr = desired absolute accuracy of factor
c                Routine will halt as soon as either rerr or aerr
c                error test passes.  To suppress one test or the other,
c                set rerr or aerr to a negative number.
c
      if (a.eq.0.d0) then
        betaff=1.d0/a
        goto 320
      endif
c        starting values
      betaff=1.d0
      bne=1.d0
      bde=1.d0
      bno=1.d0
      if (x.le.0.5d0) then
        bdo=1.d0-x*(a+b)/(a+1.d0)
      elseif (x.gt.0.5d0) then
        bdo=(-b+1.d0+xp*(a+b))/(a+1.d0)
      endif
      if (bdo.ne.0.d0) then
        bne=bne/bdo
        bde=bde/bdo
        bno=bno/bdo
        bdo=1.d0
        betaff=bno
c       be=bne/bde
        if (abs(betaff).gt.dflmax()) goto 320
        if ((.not.betaff.ge.0.d0).and.(.not.betaff.le.0.d0)) goto 320
      endif
      imax=0
      i=0
c        repeated fraction loop
  250 i=i+1
c        repeated fraction is the same for both x.le.0.5 and x.gt.0.5,
c        but it's written differently to avoid near cancellations
        if (x.le.0.5d0) then
          r=x*i/(a+2*i-1)*(b-i)/(a+2*i)
          bne=bno+r*bne
          bde=bdo+r*bde
          r=-x*(a+i)/(a+2*i)*((a+b)+i)/(a+2*i+1)
          bno=bne+r*bno
          bdo=bde+r*bdo
        elseif (x.gt.0.5d0) then
          r=x*i/(a+2*i-1)*(b-i)/(a+2*i)
          dn=r*bne
          dd=r*bde
          bne=bno+dn
          bde=bdo+dd
          r=((a+i)/(a+2*i)*((-b+2*i+1)+xp*((a+b)+i))
     &      +i*(i+1)/(a+2*i))/(a+2*i+1)
          bno=dn+r*bno
          bdo=dd+r*bdo
        endif
c        test for convergence
        if (bdo.ne.0.d0) then
          if (bno.eq.0.d0) then
            err=betaff-bno/bdo
          elseif (bno.ne.0.d0) then
            err=betaff/(bno/bdo)-1.d0
          endif
c         if (bne.eq.0.d0) then
c           ee=be-bne/bde
c         elseif (bne.ne.0.d0) then
c           ee=be/(bne/bde)-1.d0
c         endif
c         write (*,'(i4,5g13.6,i4)')
c    *      i,bne/bde/a,bno/bdo/a,bde/bdo,err,ee,imax
          bne=bne/bdo
          bde=bde/bdo
          bno=bno/bdo
          bdo=1.d0
          err=abs(err)
          if (err.lt.rerr.or.err*abs(bno/a).lt.aerr) goto 260
          if (imax.gt.0) then
            imax=imax-1
            if (imax.eq.0) goto 260
          endif
          if (err.lt.8.d0*rerr.or.err*abs(bno/a).lt.8.d0*aerr) then
            if (imax.eq.0.or.imax.gt.5) imax=5
          endif
          betaff=bno
c         be=bne/bde
          if (abs(betaff).gt.dflmax()) goto 320
          if ((.not.betaff.ge.0.d0).and.(.not.betaff.le.0.d0)) goto 320
        endif
      goto 250
c        done
  260 betaff=bno/a
c     print *,'betaff: it=',i,' x,betaff=',x,betaff
      return
c        exit here if factor is infinite
  320 return
      end
c
c-----------------------------------------------------------------------
      real*8 function betafs(x,a,b,na,lerr)
      integer na
      real*8 x,a,b,lerr
c
c        externals
      real*8 expf
c        local (automatic) variables
      integer i,ii,il,it,itl
      real*8 ai,ail,c,d,l
c *
c * Series solution of Incomplete Beta function,
c *                                                        2
c *                         a   1   (1-b) x    (1-b)(2-b) x
c *    betafs = B(x,a,b) = x  [ - + -------- + ------------- + ... ]
c *                             a   (a+1)        (a+2) 2!
c *
c *                              (1-b)_n
c * If na >= 0, then a constant ---------- is dropped,
c *                             (a+na) na!
c * so preserving accuracy when taking differences of Beta functions,
c * as in B(h,a,b)-B(x,a,b).
c *
c * If a is a negative integer with a+na = 0, then the effect
c * of subtracting the constant is to replace the divergent term
c *                 x^(a+na)      x^(a+na) - 1
c * proportional to -------  with ------------ = ln(x) .
c *                  a+na             a+na
c *
c * Used by real*8 function betafp.
c *
#ifndef EXP_IS_BAD
      expf(c)=exp(c)
#endif
c
      if (x.eq.0.d0) then
        c=0.d0
        if (a.le.0.d0) then
          c=-1.d0/c
        endif
      else
        l=log(x)
c        number of iterations to achieve log10(error) better than lerr
        it=4+lerr/log10(max(x,abs(x*(b-4.d0)/4.d0)))
c        if definite integral, make sure iterations include largest term
        if (na.lt.0.and.it.le.nint(a)) it=nint(a)+1
c        if b is a positive integer, series is finite
        if (b.gt.0.d0.and.b.eq.aint(b)
     &    .and.(it.le.3.or.it.gt.int(b))) it=int(b)
        itl=0
        c=0.d0
        do 150 ii=1,it
          i=it-ii
c        logarithmic term in term-by-term integration
          if (i.eq.na) then
            if (a+na.eq.0.d0) then
              d=l
            elseif (abs((a+na)*l).lt.0.125d0) then
              itl=1+lerr/log10(abs((a+na)*l))
              d=l
              do 130 il=2,itl
                ail=itl+2-il
                d=-d*(a+na)*l/ail+l
  130         continue
            else
              d=(1.d0-expf(-(a+na)*l))/(a+na)
            endif
c        normal term
          else
            d=1.d0/(a+i)
          endif
          ai=i+1
          c=-c*x*(b-ai)/ai+d
  150   continue
  160   c=c*expf(a*l)
      endif
      betafs=c
c     print *,'betafs: it,itl=',it,itl,' x,betafs=',x,betafs
      return
      end
c
c-----------------------------------------------------------------------
      subroutine betafh(aa,bb,h,hp)
      real*8 aa,bb,h,hp
c *
c * h to split B(1,aa,bb) = B(h,aa,bb) + B(1-h,bb,aa)
c *
c * h=(aa+1)/(aa+bb+2)
c *
c  Input: aa, bb = parameters of Beta function
c Output: h = split point
c         hp = 1-h
c
c        done as follows to maintain maximum precision
      if (aa.ge.bb) then
        hp=(bb+1.d0)/(aa+bb+2.d0)
        h=1.d0-hp
        if (h.lt.1.d0) hp=1.d0-h
      elseif (aa.le.bb) then
        h=(aa+1.d0)/(aa+bb+2.d0)
        hp=1.d0-h
        if (hp.lt.1.d0) h=1.d0-hp
      endif
      return
      end
c
c-----------------------------------------------------------------------
      real*8 function betaft(b)
      real*8 b
c
c        parameters
      integer NP
      parameter (NP=30)
      real*8 PI
      parameter (PI=3.141592653589793238462643383279502884197d0)
c        data variables
      real*8 p(NP)
c        local (automatic) variables
      integer i
      real*8 bb,s
c *
c * pi/tan(pi b) - 1/b
c * Needed to compute Incomplete Beta function B(x,a,b)
c * when a is a negative integer, a = 0, -1, -2...
c *
c        powers of b, b^3, ..., b^59 in series expansion of
c        1/b - pi/tan(pi b) - 2 b/(1-b^2)
      data p /
     &  1.2898681336964528729d0,1.6464646742227638303d-1,
     &  3.468612396889827943d-2,8.154712395888678757d-3,
     &  1.9891502556361706743d-3,4.921731066160965973d-4,
     &  1.2249627011740965852d-4,3.0564518817303743465d-5,
     &  7.634586529999679713d-6,1.9079240677455922263d-6,
     &  4.769010054554659800d-7,1.1921637810251895922d-7,
     &  2.9803109656730082469d-8,7.450668049576914110d-9,
     &  1.8626548648393363657d-9,4.656623667353010984d-10,
     &  1.1641544175805401778d-10,2.9103843782083968472d-11,
     &  7.275959094757302380d-12,1.8189895680527778565d-12,
     &  4.547473691649305030d-13,1.1368683975255171219d-13,
     &  2.8421709656063213540d-14,7.105427382674227347d-15,
     &  1.7763568421861631806d-15,4.440892101596083968d-16,
     &  1.1102230249690962487d-16,2.7755575619450465526d-17,
     &  6.938893904331845249d-18,1.7347234760239867457d-18/
c
      if (abs(b).ge..25d0) then
        betaft=PI/tan(PI*(b-anint(b)))-1.d0/b
      else
        bb=b*b
        s=0.d0
        do i=NP,1,-1
          s=s*bb+p(i)
        enddo
        s=(s+2.d0/(1.d0-bb))*b
        betaft=-s
      endif
      return
      end
c
c-----------------------------------------------------------------------
cp    real*8 function log1p(x)
cp    real*8 x
c
c        data variables
cp    real*8 lerr
c        local (automatic) variables
cp    integer i,it
cp    real*8 ai,c
c *
c * log1p(x) = log(1+x)
c *
cp    data lerr /-16.d0/
c
cp    if (x.eq.0.d0) then
cp      c=0.d0
cp    elseif (abs(x).lt.0.5d0) then
cp      it=1+lerr/log10(abs(x))
cp      ai=it
cp      c=1.d0/ai
cp      do 150 i=1,it-1
cp        ai=it-i
cp        c=-c*x+1.d0/ai
cp150   continue
cp      c=c*x
cp    else
cp      c=log(1.d0+x)
cp    endif
cp    log1p=c
cp    return
cp    end
c
