c=======================================================================
c growl
c    computes the growth rate g = D/a
c    and logarithmic derivative f = dlnD/dlna
c    of the growing mode in expanding Universes
c    containing only pressureless matter & vacuum with densities
c    Omega_m and Omega_l relative to the critical density.
c    Works for arbitrary values of Omega_m and Omega_l.
c
c Caveat:
c Loses accuracy for large values of Omega_m and/or |Omega_l|.
c If this is a problem, then increasing itmax, the maximum number of
c terms in the series expansion, (default itmax=1000) may help.
c However, increasing itmax increases the risk of instability in
c recursion of Beta functions near points of neutral stability.
c For example, the region near Einstein's loiter point at
c Omega_m = 2 Omega_l -> large is problematic.
c
c Set verbose=2 in the data statement to turn on reporting of error
c estimates.  The error estimate is computed as the ratio of the last
c (smallest) term in the series to the net value.  Thus the error is
c liable to be underestimated if the series is converging slowly,
c as occurs when Omega_m or |Omega_l| is large.  A large number of
c terms in the series indicates slow convergence.
c 
c  Input: omegam = Omega in matter.
c         omegal = Omega in cosmological constant (vacuum).
c
c Output: g = D/a, normalized to g -> 1 at a -> 0.
c         f = dlnD/dlna.
c         ok = .true. if input parameters were physically valid;
c              .false. if not.
c
c The growl package is available at
c http://casa.colorado.edu/~ajsh/growl/
c
c A. J. S. Hamilton (2000), MNRAS, submitted.
c Andrew.Hamilton@colorado.edu
c
c---Start of revision history-------------------------------------------
c
c 6 June 2000:
c Original version.
c
c 10 June 2000:
c Tweaked.  More comments.
c
c 12 June 2000:
c Fixed bug at Omega_m = 1, Omega_l = 0
c pointed out by Matthew Graham <m.j.graham@ic.ac.uk>.
c
c 13 June 2000:
c Fixed bug at tiny Omega_m.
c g is now accurate even at hideously tiny nonzero Omega_m.
c However, f= - (1/2) Omega_m + Omega_l - 1 + (5/2) Omega_m/g
c remains accurate only to rounding error (abs error of about 10^-16).
c
c 27 June 2000:
c Added more dire warnings in comments about things going bad at large
c Omega_m and/or |Omegal|.
c
c---End of revision history---------------------------------------------
c
c=======================================================================
      subroutine growl(omegam,omegal,g,f,ok)
      logical ok
      real*8 omegam,omegal,g,f
c
c        parameters
      integer K,L,M
      parameter (K=0,L=1,M=2)
      real*8 HALF,THIRD,TWOTHD,FOURTHD,FIVSIX,SEVSIX
      parameter (HALF=1.d0/2.d0,THIRD=1.d0/3.d0,TWOTHD=2.d0/3.d0,
     &  FOURTHD=4.d0/3.d0,FIVSIX=5.d0/6.d0,SEVSIX=7.d0/6.d0)
c When recursing B(x,a,b) by a -> a + m, b -> b + n
c with m and n positive or negative integers,
c neutral stability occurs asymptotically (i.e. after many recursions)
c at roots of
c x^m (1-x)^n = +- m^m n^n / (m+n)^(m+n)
      real*8 XKP
      parameter (XKP=1.d0/3.d0)
c        XKM = root of (4 x^3 - 12 x^2 + 39 x - 4 = 0)
c            = 1 + (3/2) [(sqrt(2) - 1)^(1/3) - (sqrt(2) - 1)^(-1/3)]
      real*8 XKM
      parameter (XKM=.1058925430250177d0)
      real*8 XLP
      parameter (XLP=.75d0)
c        XLM = root of (4 x^3 + 27 x - 27 = 0)
c            = (3/2) ((Sqrt[2] + 1)^(1/3) - (Sqrt[2] + 1)^(-1/3))
      real*8 XLM
      parameter (XLM=.8941074569749823d0)
      real*8 BIG
      parameter (BIG=1.d10)
c        externals
      real*8 dflmax
      real*8 betafp
c        data variables
      integer itmax,verbose
      real*8 prec
c        local (automatic) variables
      integer i,ib,it,j,method
      real*8 aeq,a(3),b(3),bet(3),d,dg,err,omegak,omk,oml,omm,omt,
     &  q,qit,x,xy(3),y,z,zz
c *
c * Growth rate
c * g = D/a
c * and logarithmic derivative
c * f = dlnD/dlna
c * of the growing mode in a Universe containing matter & vacuum.
c *
c  Input: omegam = Omega in matter.
c         omegal = Omega in cosmological constant (vacuum).
c Output: g = D/a, normalized to g -> 1 at a -> 0.
c         f = dlnD/dlna.
c         ok = .true. if input parameters were physically valid;
c              .false. if not.
c
c        verbose: 0 = quiet
c                 1 = advise bad inputs
c                 2 = report estimated uncertainty in value
c                 3 = debug
c                 4 = debug + choose method
      data verbose /1/
c        precision with which g, f are desired
      data prec /1.d-16/
c        maximum number of terms in expansion
c Number of terms can grow large for large values of omegam and/or omegal.
c If itmax is set to some large number, then recursion of Beta functions
c can start to become unstable near neutral points of stability
c for large values of omegam and/or omegal.
      data itmax /1000/
c
c--------omega curvature
      omegak=1.d0-omegam-omegal
c--------check inputs ok
      ok=.false.
      g=0.d0
      f=0.d0
c        omega matter density should be >= 0
      if (omegam.lt.0.d0) then
        goto 320
c        curvature should not be too closed
      elseif (omegal.gt.0.d0.and.omegak.lt.min(-1.5d0*omegam,
     &  -(6.75d0*omegam**2*omegal)**THIRD)) then
        goto 330
      endif
c--------zero Omega_m
c Limiting case of vanishing matter density.
c Physically, structure cannot grow in a pure vacuum Universe.
c Requires special treatment because other formulae are singular here.
      if (omegam.eq.0.d0) then
        g=0.d0
        f=0.d0
        goto 200
c--------flat matter-dominated, Omega_m = 1, Omega_l = 0
c Requires special treatment because other formulae are singular here.
      elseif (omegam.eq.1.d0.and.omegam+abs(omegal).eq.1.d0) then
        g=1.d0
        f=1.d0
        goto 200
      endif
c--------decide which method to use
c        one of omm, omk, oml must be < 1
      if (omegam.eq.1.d0) then
        omm=BIG
      else
        omm=abs(omegam/(1.d0-omegam))
      endif
      if (omegam+omegal.eq.0.d0) then
        omk=BIG
      else
        omk=abs(omegak/(omegam+omegal))
      endif
      if (omegal.eq.1.d0) then
        oml=BIG
      else
        oml=abs(omegal/(1.d0-omegal))
      endif
c        smallest of omm, omk, oml
      if (omegak.gt.0.d0.and.omm.lt.omk.and.omm.lt.oml) then
        method=M
      elseif (oml.lt.omk) then
        method=L
      else
        method=K
      endif
c--------refine choice of method
      if (verbose.ge.4) then
        it=2
      else
        it=1
      endif
      do i=1,it
        if (i.eq.2) then
c        debug option to use alternative method
          print *,'method =',method,'; enter method: 0=K, 1=L, +2=M'
          read (*,*,end=200,err=200) method
          if (method.lt.0.or.method.gt.3) goto 200
        endif
c        default
        omm=omegam
        omk=omegak
        oml=omegal
c........method M: split integral into two parts
        if (method/M.eq.1) then
          if (omegal.ge.0.d0) then
c        choose the later epoch between m=k or m=l equality
            if (omegam/omegak.gt.(omegam/omegal)**THIRD) then
c method K always beats M+K (see paper)
c             method=M+K
c             aeq=omegam/omegak
              method=K
            else
              method=M+L
              aeq=(omegam/omegal)**THIRD
            endif
          else
c        epoch aeq where m/(1-m) = - l/(1-l)
            method=M+L
c        1st guess
            aeq=(-omegam/omegal)**THIRD
c        Newton improved guess (aeq need not be perfect)
            aeq=aeq*(4.d0/3.d0*omegam-omegak*aeq)
     &        /(2.d0*omegam-omegak*aeq)
          endif
c        omega's at aeq
          if (method/M.eq.1) then
            omm=omegam/aeq**3
            omk=omegak/aeq**2
            oml=omegal
            omt=omm+omk+oml
            omm=omm/omt
            omk=omk/omt
            oml=oml/omt
          endif
        endif
        if (verbose.ge.3) then
          print *,'++++++++++++++++++++'
          print *,'method',method,
     &      ' Om_m =',omm,' Om_k =',omk,' Om_l =',oml
          if (method/M.eq.1) print *,'aeq =',aeq
        endif
      enddo
c--------method K (small Omega_k), Omega_l > 0
      if (mod(method,M).eq.K.and.omegal.gt.0.d0) then
        if (verbose.ge.3) then
          print *,'--------------------'
          print *,'method K, Omega_l > 0'
        endif
        x=oml/(omm+oml)
        y=omm/(omm+oml)
        z=omk/(omm**2*oml)**THIRD
        zz=omk**3/(omm**2*oml)
        if (verbose.ge.3) print *,'x, y, z, zz =',x,y,z,zz
        if (x.lt.0.d0.or.x.gt.1.d0) goto 340
c........recurse Beta functions from 1st three terms upward
c Recursion has been tested.  I would have expected recursion to be
c upwardly stable both sides of the neutral point XKP = 1/3, but in
c fact the stable direction changes at or near the neutral point.
c Examples:
c  x =approx .28   for omegam=100, omegal=39,      with itmax=10^4;
c  x =approx .3325 for omegam=1e7, omegal=4980000, with itmax=10^7.
        if (x.gt.XKP.or.z.eq.0.d0) then
          if (z.eq.0.d0) then
            qit=0.d0
            it=1
          elseif (omm.ge.2.d0*oml) then
            qit=omk/(omm+oml)
            it=1+log(prec)/log(abs(qit))
            if (it.le.1) it=2
            if (verbose.ge.3) print *,'Om_k/(Om_m+Om_l) =',qit,
     &        '; expect',it,' terms'
          else
            qit=2.d0**TWOTHD/3.d0*z
            it=1+log(prec)/log(abs(qit))
            if (it.le.1) it=2
            if (verbose.ge.3) print *,
     &        '[(4/27) Om_k^3/(Om_m^2 Om_l)]^(1/3) =',qit,
     &        '; expect',it,' terms'
          endif
          if (.not.(it.le.itmax)) then
            it=itmax
            if (verbose.ge.3) print *,'will do only',it,' terms'
          endif
          if (verbose.ge.3) print *,'recurse from 1st 3 terms upward'
          a(1)=FIVSIX
          a(2)=SEVSIX
          a(3)=1.5d0
          b(1)=TWOTHD
          b(2)=FOURTHD
          b(3)=2.d0
          xy(1)=x**a(1)*y**b(1)
          xy(2)=x**a(2)*y**b(2)*z
          xy(3)=x**1.5d0*y**2*z**2
          bet(1)=betafp(x,y,a(1),b(1))
          bet(2)=betafp(x,y,a(2),b(2))*z
c        3rd incomplete Beta function is analytic
          bet(3)=2./15.d0*x**1.5d0*(5.d0-3.d0*x)*z**2
          err=prec*abs(bet(1))
          q=1.d0
          g=0.d0
          do i=1,it
            ib=1+mod(i-1,3)
            d=bet(ib)*q
            g=g+d
            if (verbose.ge.3) then
              if (i.le.10.or.i.gt.it-10.or.abs(d).le.err) then
                print *,i,' d =',d,
     &            ' B(',a(ib),',',b(ib),') z^(i-1) =',bet(ib),
     &            ' =',betafp(x,y,a(ib),b(ib))*z**(i-1)
              elseif (i.eq.11) then
                print *,'...'
              endif
            endif
            if (abs(d).le.err.or.i.eq.it) goto 110
            q=-q*(dble(i)+HALF)/dble(i)
c        recurse B_x(a,b) -> B_x(a,b+1)
            bet(ib)=(bet(ib)*b(ib)+xy(ib))/(a(ib)+b(ib))
            b(ib)=b(ib)+1.d0
            xy(ib)=xy(ib)*y
c        recurse B_x(a,b) -> B_x(a+1,b)
            bet(ib)=(bet(ib)*a(ib)-xy(ib))/(a(ib)+b(ib))
            a(ib)=a(ib)+1.d0
            xy(ib)=xy(ib)*x
c        recurse B_x(a,b) -> B_x(a,b+1)
            bet(ib)=(bet(ib)*b(ib)+xy(ib))/(a(ib)+b(ib))
            b(ib)=b(ib)+1.d0
            xy(ib)=xy(ib)*y
c        incorporate z factor into B, to avoid large/small numbers
            bet(ib)=bet(ib)*zz
            xy(ib)=xy(ib)*zz
          enddo
  110     if (qit.eq.0.d0) then
            err=0.d0
          else
            err=d/g
          endif
c        notice omega's, not om's used here, so integrals can be summed
          g=g*FIVSIX*omegam**THIRD/omegal**FIVSIX
          if (verbose.ge.2) print *,'method K Om_l > 0,',
     &      i,' terms; g =',g,' relerr =',err
c........recurse Beta functions from last three terms downward
        else
          if (omm.ge.2.d0*oml) then
            qit=omk/(omm+oml)
            it=1+log(prec)/log(abs(qit))
            if (it.le.1) it=2
            if (verbose.ge.3) print *,'Om_k/(Om_m+Om_l) =',qit,
     &        '; use',it,' terms'
          else
            qit=2.d0**TWOTHD/3.d0*z
            it=1+log(prec)/log(abs(qit))
            if (it.le.1) it=2
            if (verbose.ge.3) print *,
     &        '[(4/27) Om_k^3/(Om_m^2 Om_l)]^(1/3) =',qit,
     &        '; use',it,' terms'
          endif
          if (.not.(it.le.itmax)) then
            it=itmax
            if (verbose.ge.3) print *,'will do only',it,' terms'
          endif
          if (verbose.ge.3) print *,'recurse from last 3 terms downward'
          do ib=1,min(3,it)
            i=it-ib
            a(ib)=FIVSIX+THIRD*i
            b(ib)=TWOTHD+TWOTHD*i
            xy(ib)=x**FIVSIX*y**TWOTHD*((x*y**2)**THIRD*z)**i
            bet(ib)=betafp(x,y,a(ib),b(ib))*z**i
c        deal with NaN or Infinity
            if ((.not.(bet(ib).le.0.d0.or.bet(ib).ge.0.d0))
     &        .or.abs(bet(ib)).gt.dflmax()) then
              if (verbose.ge.3) print *,
     &          i,' starting B(',a(ib),',',b(ib),') z^i =',bet(ib)
              bet(ib)=0.d0
            endif
          enddo
          g=0.d0
          err=0.d0
          do i=it,1,-1
            ib=1+mod(it-i,3)
            d=-g*(dble(i)+HALF)/dble(i)
            g=d+bet(ib)
            if (err.eq.0.d0) then
              err=bet(ib)
            else
              err=-err*(dble(i)+HALF)/dble(i)
            endif
            if (verbose.ge.3) then
              if (i.le.10.or.i.gt.it-10) then
                print *,i,' d =',d,
     &            ' B(',a(ib),',',b(ib),') z^(i-1) =',bet(ib),
     &            ' =',betafp(x,y,a(ib),b(ib))*z**(i-1)
              elseif (i.eq.11) then
                print *,'...'
              endif
            endif
c        incorporate z factor into B, to avoid large/small numbers
            bet(ib)=bet(ib)/zz
            xy(ib)=xy(ib)/zz
c        recurse B_x(a,b) -> B_x(a,b-1)
            b(ib)=b(ib)-1.d0
            xy(ib)=xy(ib)/y
            bet(ib)=(bet(ib)*(a(ib)+b(ib))-xy(ib))/b(ib)
c        recurse B_x(a,b) -> B_x(a-1,b)
            a(ib)=a(ib)-1.d0
            xy(ib)=xy(ib)/x
            bet(ib)=(bet(ib)*(a(ib)+b(ib))+xy(ib))/a(ib)
c        recurse B_x(a,b) -> B_x(a,b-1)
            b(ib)=b(ib)-1.d0
            xy(ib)=xy(ib)/y
            bet(ib)=(bet(ib)*(a(ib)+b(ib))-xy(ib))/b(ib)
          enddo
          if (qit.eq.0.d0) then
            err=0.d0
          else
            err=err/g
          endif
c        notice omega's, not om's used here, so integrals can be summed
          g=g*FIVSIX*omegam**THIRD/omegal**FIVSIX
          if (verbose.ge.2) print *,'method K Om_l > 0,',
     &      it,' terms; g =',g,' relerr =',err
        endif
c--------method K (small Omega_k), Omega_l < 0
      elseif (mod(method,M).eq.K.and.omegal.lt.0.d0) then
        if (verbose.ge.3) then
          print *,'--------------------'
          print *,'method K, Omega_l < 0'
        endif
        x=-oml/omm
        y=(omm+oml)/omm
        z=omk/(omm**2*(-oml))**THIRD
        zz=omk**3/(omm**2*(-oml))
        if (verbose.ge.3) print *,'x, y, z, zz =',x,y,z,zz
        if (x.lt.0.d0.or.x.gt.1.d0) goto 340
c........recurse Beta functions from 1st three terms upward
        if (x.gt.XKM.or.z.eq.0.d0) then
          if (z.eq.0.d0) then
            qit=0.d0
            it=1
          else
            qit=omk/(omm+oml)
            it=1+log(prec)/log(abs(qit))
            if (it.le.1) it=2
          endif
          if (verbose.ge.3) print *,'Om_k/(Om_m+Om_l) =',qit,
     &      '; expect',it,' terms'
          if (.not.(it.le.itmax)) then
            it=itmax
            if (verbose.ge.3) print *,'will do only',it,' terms'
          endif
          if (verbose.ge.3) print *,'recurse from 1st 3 terms upward'
          a(1)=FIVSIX
          a(2)=SEVSIX
          a(3)=1.5d0
          b(1)=-.5d0
          b(2)=-1.5d0
          b(3)=-2.5d0
          xy(1)=x**a(1)*y**b(1)
          xy(2)=x**a(2)*y**b(2)*z
          xy(3)=x**a(3)*y**b(3)*z**2
          bet(1)=betafp(x,y,a(1),b(1))
          bet(2)=betafp(x,y,a(2),b(2))*z
          bet(3)=betafp(x,y,a(3),b(3))*z**2
          err=prec*abs(bet(1))
          q=1.d0
          g=0.d0
          do i=1,it
            ib=1+mod(i-1,3)
            d=bet(ib)*q
            g=g+d
            if (verbose.ge.3) then
              if (i.le.10.or.i.gt.it-10.or.abs(d).le.err) then
                print *,i,' d =',d,
     &            ' B(',a(ib),',',b(ib),') z^(i-1) =',bet(ib),
     &            ' =',betafp(x,y,a(ib),b(ib))*z**(i-1)
              elseif (i.eq.11) then
                print *,'...'
              endif
            endif
            if (abs(d).le.err.or.i.eq.it) goto 120
            q=-q*(dble(i)+HALF)/dble(i)
c        recurse B_x(a,b) -> B_x(a,b-1)
            b(ib)=b(ib)-1.d0
            xy(ib)=xy(ib)/y
            bet(ib)=(bet(ib)*(a(ib)+b(ib))-xy(ib))/b(ib)
c        recurse B_x(a,b) -> B_x(a+1,b-1)
            b(ib)=b(ib)-1.d0
            xy(ib)=xy(ib)/y
            bet(ib)=(bet(ib)*a(ib)-xy(ib))/b(ib)
            a(ib)=a(ib)+1.d0
            xy(ib)=xy(ib)*x
c        recurse B_x(a,b) -> B_x(a,b-1)
            b(ib)=b(ib)-1.d0
            xy(ib)=xy(ib)/y
            bet(ib)=(bet(ib)*(a(ib)+b(ib))-xy(ib))/b(ib)
c        incorporate z factor into B, to avoid large/small numbers
            bet(ib)=bet(ib)*zz
            xy(ib)=xy(ib)*zz
          enddo
  120     if (qit.eq.0.d0) then
            err=0.d0
          else
            err=d/g
          endif
c        notice omega's, not om's used here, so integrals can be summed
          g=g*FIVSIX*omegam**THIRD/(-omegal)**FIVSIX
          if (verbose.ge.2) print *,'method K Om_l < 0,',
     &      i,' terms; g =',g,' relerr =',err
c........recurse Beta functions from last three terms downward
        else
          qit=omk/(omm+oml)
          it=1+log(prec)/log(abs(qit))
          if (it.le.1) it=2
          if (verbose.ge.3) print *,'Om_k/(Om_m+Om_l) =',qit,
     &      '; use',it,' terms'
          if (.not.(it.le.itmax)) then
            it=itmax
            if (verbose.ge.3) print *,'will do only',it,' terms'
          endif
          if (verbose.ge.3) print *,'recurse from last 3 terms downward'
          do ib=1,min(3,it)
            i=it-ib
            a(ib)=FIVSIX+THIRD*i
            b(ib)=-HALF-i
            xy(ib)=x**FIVSIX/sqrt(y)*(x**THIRD/y*z)**i
            bet(ib)=betafp(x,y,a(ib),b(ib))*z**i
c        deal with NaN or Infinity
            if ((.not.(bet(ib).le.0.d0.or.bet(ib).ge.0.d0))
     &        .or.abs(bet(ib)).gt.dflmax()) then
              if (verbose.ge.3) print *,
     &          i,' starting B(',a(ib),',',b(ib),') z^i =',bet(ib)
              bet(ib)=0.d0
            endif
          enddo
          g=0.d0
          err=0.d0
          do i=it,1,-1
            ib=1+mod(it-i,3)
            d=-g*(dble(i)+HALF)/dble(i)
            g=d+bet(ib)
            if (err.eq.0.d0) then
              err=bet(ib)
            else
              err=-err*(dble(i)+HALF)/dble(i)
            endif
            if (verbose.ge.3) then
              if (i.le.10.or.i.gt.it-10) then
                print *,i,' d =',d,
     &            ' B(',a(ib),',',b(ib),') z^(i-1) =',bet(ib),
     &            ' =',betafp(x,y,a(ib),b(ib))*z**(i-1)
              elseif (i.eq.11) then
                print *,'...'
              endif
            endif
c        incorporate z factor into B, to avoid large/small numbers
            bet(ib)=bet(ib)/zz
            xy(ib)=xy(ib)/zz
c        recurse B_x(a,b) -> B_x(a,b+1)
            bet(ib)=(bet(ib)*b(ib)+xy(ib))/(a(ib)+b(ib))
            b(ib)=b(ib)+1.d0
            xy(ib)=xy(ib)*y
c        recurse B_x(a,b) -> B_x(a-1,b+1)
            a(ib)=a(ib)-1.d0
            xy(ib)=xy(ib)/x
            bet(ib)=(bet(ib)*b(ib)+xy(ib))/a(ib)
            b(ib)=b(ib)+1.d0
            xy(ib)=xy(ib)*y
c        recurse B_x(a,b) -> B_x(a,b+1)
            bet(ib)=(bet(ib)*b(ib)+xy(ib))/(a(ib)+b(ib))
            b(ib)=b(ib)+1.d0
            xy(ib)=xy(ib)*y
          enddo
          if (qit.eq.0.d0) then
            err=0.d0
          else
            err=err/g
          endif
c        notice omega's, not om's used here, so integrals can be summed
          g=g*FIVSIX*omegam**THIRD/(-omegal)**FIVSIX
          if (verbose.ge.2) print *,'method K Om_l < 0,',
     &      it,' terms; g =',g,' relerr =',err
        endif
c--------method L (small Omega_l), Omega_k > 0
      elseif (mod(method,M).eq.L.and.omegak.gt.0.d0) then
        if (verbose.ge.3) then
          print *,'--------------------'
          print *,'method L, Omega_k > 0'
        endif
        x=omk/(1.d0-oml)
        y=omm/(1.d0-oml)
        z=omm**2*oml/omk**3
        if (verbose.ge.3) print *,'x, y, z =',x,y,z
        if (x.lt.0.d0.or.x.gt.1.d0) goto 340
c........recurse Beta functions from 1st term upward
        if (x.gt.XLP.or.z.eq.0.d0) then
          if (z.eq.0.d0) then
            qit=0.d0
            it=1
          else
            qit=oml/(1.d0-oml)
            it=1+log(prec)/log(abs(qit))
            if (it.le.1) it=2
          endif
          if (verbose.ge.3) print *,'Om_l/(Om_m+Om_k) =',qit,
     &      '; expect',it,' terms'
          if (.not.(it.le.itmax)) then
            it=itmax
            if (verbose.ge.3) print *,'will do only',it,' terms'
          endif
          if (verbose.ge.3) print *,'recurse from 1st term upward'
          a(1)=2.5d0
          b(1)=-1.d0
          xy(1)=x**a(1)/y
          bet(1)=betafp(x,y,a(1),b(1))
          err=prec*abs(bet(1))
          q=1.d0
          g=0.d0
          do i=1,it
            d=bet(1)*q
            g=g+d
            if (verbose.ge.3) then
              if (i.le.10.or.i.gt.it-10.or.abs(d).le.err) then
                print *,i,' d =',d,' g =',g,
     &            ' B(',a(1),',',b(1),') z^(i-1) =',bet(1),
     &            ' =',betafp(x,y,a(1),b(1))*z**(i-1)
              elseif (i.eq.11) then
                print *,'...'
              endif
            endif
            if (abs(d).le.err.or.i.eq.it) goto 130
            q=-q*(dble(i)+HALF)/dble(i)
c        recurse B_x(a,b) -> B_x(a+1,b-1)
            b(1)=b(1)-1.d0
            xy(1)=xy(1)/y
            bet(1)=(bet(1)*a(1)-xy(1))/b(1)
            a(1)=a(1)+1.d0
            xy(1)=xy(1)*x
c        recurse B_x(a,b) -> B_x(a+1,b-1)
            b(1)=b(1)-1.d0
            xy(1)=xy(1)/y
            bet(1)=(bet(1)*a(1)-xy(1))/b(1)
            a(1)=a(1)+1.d0
            xy(1)=xy(1)*x
c        recurse B_x(a,b) -> B_x(a+1,b)
            bet(1)=(bet(1)*a(1)-xy(1))/(a(1)+b(1))
            a(1)=a(1)+1.d0
            xy(1)=xy(1)*x
c        incorporate z factor into B, to avoid large/small numbers
            bet(1)=bet(1)*z
            xy(1)=xy(1)*z
          enddo
  130     if (qit.eq.0.d0) then
            err=0.d0
          else
            err=d/g
          endif
c        notice omega's, not om's used here, so integrals can be summed
          g=g*2.5d0*omegam**2/omegak**2.5d0
          if (verbose.ge.2) print *,'method L Om_k > 0,',
     &      i,' terms; g =',g,' relerr =',err
c........recurse Beta functions from it'th term downward
        else
          qit=oml/(1.d0-oml)
          it=1+log(prec)/log(abs(qit))
          if (it.le.1) it=2
          if (verbose.ge.3) print *,'Om_l/(Om_m+Om_k) =',qit,
     &      '; use',it,' terms'
          if (.not.(it.le.itmax)) then
            it=itmax
            if (verbose.ge.3) print *,'will do only',it,' terms'
          endif
          if (verbose.ge.3)
     &      print *,'recurse from',it+1,'''th term downward'
          a(1)=2.5d0+3.d0*it
          b(1)=-1.d0-2.d0*it
          xy(1)=x**2.5d0/y*(x**3*z/y**2)**it
          bet(1)=betafp(x,y,a(1),b(1))*z**it
c        deal with NaN or Infinity
          if ((.not.(bet(1).le.0.d0.or.bet(1).ge.0.d0))
     &      .or.abs(bet(1)).gt.dflmax()) then
            if (verbose.ge.3) print *,
     &        it,' starting B(',a(1),',',b(1),') z^i =',bet(1)
            bet(1)=0.d0
          endif
          g=bet(1)
          err=bet(1)
          do i=it,1,-1
c        incorporate z factor into B, to avoid large/small numbers
            bet(1)=bet(1)/z
            xy(1)=xy(1)/z
c        recurse B_x(a,b) -> B_x(a-1,b)
            a(1)=a(1)-1.d0
            xy(1)=xy(1)/x
            bet(1)=(bet(1)*(a(1)+b(1))+xy(1))/a(1)
c        recurse B_x(a,b) -> B_x(a-1,b+1)
            a(1)=a(1)-1.d0
            xy(1)=xy(1)/x
            bet(1)=(bet(1)*b(1)+xy(1))/a(1)
            b(1)=b(1)+1.d0
            xy(1)=xy(1)*y
c        recurse B_x(a,b) -> B_x(a-1,b+1)
            a(1)=a(1)-1.d0
            xy(1)=xy(1)/x
            bet(1)=(bet(1)*b(1)+xy(1))/a(1)
            b(1)=b(1)+1.d0
            xy(1)=xy(1)*y
            d=-g*(dble(i)+HALF)/dble(i)
            g=d+bet(1)
            if (i.eq.it.and.err.eq.0.d0) then
              err=bet(1)
            else
              err=-err*(dble(i)+HALF)/dble(i)
            endif
            if (verbose.ge.3) then
              if (i.le.10.or.i.gt.it-10) then
                print *,i,' d =',d,
     &            ' B(',a(1),',',b(1),') z^(i-1) =',bet(1),
     &            ' =',betafp(x,y,a(1),b(1))*z**(i-1)
              elseif (i.eq.11) then
                print *,'...'
              endif
            endif
          enddo
          if (qit.eq.0.d0) then
            err=0.d0
          else
            err=err/g
          endif
c        notice omega's, not om's used here, so integrals can be summed
          g=g*2.5d0*omegam**2/omegak**2.5d0
          if (verbose.ge.2) print *,'method L Om_k > 0,',
     &      it,' terms; g =',g,' relerr =',err
        endif
c--------method L (small Omega_l), Omega_k < 0
      elseif (mod(method,M).eq.L.and.omegak.lt.0.d0) then
        if (verbose.ge.3) then
          print *,'--------------------'
          print *,'method L, Omega_k < 0'
        endif
        x=-omk/omm
        y=(1.d0-oml)/omm
        z=oml*omm**2/(-omk)**3
        if (verbose.ge.3) print *,'x, y, z =',x,y,z
        if (x.lt.0.d0.or.x.gt.1.d0) goto 340
c........recurse Beta functions from 1st term upward
c Numerical experiment confirms expected changeover of stability
c at x=XLM with exquisite accuracy.
        if (x.gt.XLM.or.z.eq.0.d0) then
          if (z.eq.0.d0) then
            qit=0.d0
            it=1
          else
            qit=oml/(1.d0-oml)
            it=1+log(prec)/log(abs(qit))
            if (it.le.1) it=2
          endif
          if (verbose.ge.3) print *,'Om_l/(Om_m+Om_k) =',qit,
     &      '; expect',it,' terms'
          if (.not.(it.le.itmax)) then
            it=itmax
            if (verbose.ge.3) print *,'will do only',it,' terms'
          endif
          if (verbose.ge.3) print *,'recurse from 1st term upward'
          a(1)=2.5d0
          b(1)=-.5d0
          xy(1)=x**a(1)/sqrt(y)
          bet(1)=betafp(x,y,a(1),b(1))
          err=prec*abs(bet(1))
          q=1.d0
          g=0.d0
          do i=1,it
            d=bet(1)*q
            g=g+d
            if (verbose.ge.3) then
              if (i.le.10.or.i.gt.it-10.or.abs(d).le.err) then
                print *,i,' d =',d,
     &            ' B(',a(1),',',b(1),') z^(i-1) =',bet(1),
     &            ' =',betafp(x,y,a(1),b(1))*z**(i-1)
              elseif (i.eq.11) then
                print *,'...'
              endif
            endif
            if (abs(d).le.err.or.i.eq.it) goto 140
            q=-q*(dble(i)+HALF)/dble(i)
c        recurse B_x(a,b) -> B_x(a+1,b)
            bet(1)=(bet(1)*a(1)-xy(1))/(a(1)+b(1))
            a(1)=a(1)+1.d0
            xy(1)=xy(1)*x
c        recurse B_x(a,b) -> B_x(a+1,b-1)
            b(1)=b(1)-1.d0
            xy(1)=xy(1)/y
            bet(1)=(bet(1)*a(1)-xy(1))/b(1)
            a(1)=a(1)+1.d0
            xy(1)=xy(1)*x
c        recurse B_x(a,b) -> B_x(a+1,b)
            bet(1)=(bet(1)*a(1)-xy(1))/(a(1)+b(1))
            a(1)=a(1)+1.d0
            xy(1)=xy(1)*x
c        incorporate z factor into B, to avoid large/small numbers
            bet(1)=bet(1)*z
            xy(1)=xy(1)*z
          enddo
          continue
  140     if (qit.eq.0.d0) then
            err=0.d0
          else
            err=d/g
          endif
c        notice omega's, not om's used here, so integrals can be summed
          g=g*2.5d0*omegam**2/(-omegak)**2.5d0
          if (verbose.ge.2) print *,'method L Om_k < 0,',
     &      i,' terms; g =',g,' relerr =',err
c........recurse Beta functions from it'th term downward
        else
          qit=oml/(1.d0-oml)
          it=1+log(prec)/log(abs(qit))
          if (it.le.1) it=2
          if (verbose.ge.3) print *,'Om_l/(Om_m+Om_k) =',qit,
     &      '; use',it,' terms'
          if (.not.(it.le.itmax)) then
            it=itmax
            if (verbose.ge.3) print *,'will do only',it,' terms'
          endif
          if (verbose.ge.3)
     &      print *,'recurse from',it+1,'''th term downward'
          a(1)=2.5d0+3.d0*it
          b(1)=-.5d0-it
          xy(1)=x**2.5d0/sqrt(y)*(x**3*z/y)**it
          bet(1)=betafp(x,y,a(1),b(1))*z**it
c        deal with NaN or Infinity
          if ((.not.(bet(1).le.0.d0.or.bet(1).ge.0.d0))
     &      .or.abs(bet(1)).gt.dflmax()) then
            if (verbose.ge.3) print *,
     &        it,' starting B(',a(1),',',b(1),') z^i =',bet(1)
            bet(1)=0.d0
          endif
          g=bet(1)
          err=bet(1)
          do i=it,1,-1
c        incorporate z factor into B, to avoid large/small numbers
            bet(1)=bet(1)/z
            xy(1)=xy(1)/z
c        recurse B_x(a,b) -> B_x(a-1,b)
            a(1)=a(1)-1.d0
            xy(1)=xy(1)/x
            bet(1)=(bet(1)*(a(1)+b(1))+xy(1))/a(1)
c        recurse B_x(a,b) -> B_x(a-1,b+1)
            a(1)=a(1)-1.d0
            xy(1)=xy(1)/x
            bet(1)=(bet(1)*b(1)+xy(1))/a(1)
            b(1)=b(1)+1.d0
            xy(1)=xy(1)*y
c        recurse B_x(a,b) -> B_x(a-1,b)
            a(1)=a(1)-1.d0
            xy(1)=xy(1)/x
            bet(1)=(bet(1)*(a(1)+b(1))+xy(1))/a(1)
            d=-g*(dble(i)+HALF)/dble(i)
            if (i.eq.it.and.err.eq.0.d0) then
              err=bet(1)
            else
              err=-err*(dble(i)+HALF)/dble(i)
            endif
            g=d+bet(1)
            if (verbose.ge.3) then
              if (i.le.10.or.i.gt.it-10) then
                print *,i,' d =',d,
     &            ' B(',a(1),',',b(1),') z^(i-1) =',bet(1),
     &            ' =',betafp(x,y,a(1),b(1))*z**(i-1)
              elseif (i.eq.11) then
                print *,'...'
              endif
            endif
          enddo
          if (qit.eq.0.d0) then
            err=0.d0
          else
            err=err/g
          endif
c        notice omega's, not om's used here, so integrals can be summed
          g=g*2.5d0*omegam**2/(-omegak)**2.5d0
          if (verbose.ge.2) print *,'method L Om_l < 0,',
     &      it,' terms; g =',g,' relerr =',err
        endif
      endif
c--------method M (small Omega_m), Omega_k > 0, Omega_l > 0
      if (method/M.eq.1.and.omegal.gt.0.d0) then
        do j=1,2
          if (verbose.ge.3) then
            print *,'--------------------'
            print *,'method M, Omega_l > 0, part',j
          endif
          if (j.eq.2) then
            omm=omegam
            omk=omegak
            oml=omegal
          endif
          x=omk/(1.d0-omm)
          y=oml/(1.d0-omm)
          z=omm*sqrt(oml/omk**3)
          zz=omm**2*oml/omk**3
          if (verbose.ge.3) print *,'x, y, z, zz =',x,y,z,zz
          if (x.lt.0.d0.or.x.gt.1.d0) goto 340
c........recurse Beta functions from 1st 2 terms upward
          if (x.gt.XLM.or.z.eq.0.d0) then
            if (z.eq.0.d0) then
              qit=0.d0
              it=1
            else
              qit=omm/(1.d0-omm)
              it=1+log(prec)/log(abs(qit))
              if (it.le.1) it=2
            endif
            if (verbose.ge.3) print *,'Om_m/(Om_k+Om_l) =',qit,
     &        '; expect',it,' terms'
            if (.not.(it.le.itmax)) then
              it=itmax
              if (verbose.ge.3) print *,'will do only',it,' terms'
            endif
            if (verbose.ge.3) print *,'recurse from 1st 2 terms upward'
            a(1)=1.d0
            b(1)=.5d0
            a(2)=2.5d0
            b(2)=0.d0
            xy(1)=x*sqrt(y)
            xy(2)=x**a(2)*z
            bet(1)=betafp(x,y,a(1),b(1))
            bet(2)=betafp(x,y,a(2),b(2))*z
            err=prec*abs(bet(1))
            q=1.d0
            dg=0.d0
            do i=1,it
              ib=1+mod(i-1,2)
              d=bet(ib)*q
              dg=dg+d
              if (verbose.ge.3) then
                if (i.le.10.or.i.gt.it-10.or.abs(d).le.err) then
                  print *,i,' d =',d,
     &              ' B(',a(ib),',',b(ib),') z^(i-1) =',bet(ib),
     &              ' =',betafp(x,y,a(ib),b(ib))*z**(i-1)
                elseif (i.eq.11) then
                  print *,'...'
                endif
              endif
              if (abs(d).le.err.or.i.eq.it) goto 150
              q=-q*(dble(i)+HALF)/dble(i)
c        recurse B_x(a,b) -> B_x(a+1,b)
              bet(ib)=(bet(ib)*a(ib)-xy(ib))/(a(ib)+b(ib))
              a(ib)=a(ib)+1.d0
              xy(ib)=xy(ib)*x
c        recurse B_x(a,b) -> B_x(a+1,b-1)
              b(ib)=b(ib)-1.d0
              xy(ib)=xy(ib)/y
              bet(ib)=(bet(ib)*a(ib)-xy(ib))/b(ib)
              a(ib)=a(ib)+1.d0
              xy(ib)=xy(ib)*x
c        recurse B_x(a,b) -> B_x(a+1,b)
              bet(ib)=(bet(ib)*a(ib)-xy(ib))/(a(ib)+b(ib))
              a(ib)=a(ib)+1.d0
              xy(ib)=xy(ib)*x
c        incorporate z factor into B, to avoid large/small numbers
              bet(ib)=bet(ib)*zz
              xy(ib)=xy(ib)*zz
            enddo
  150       if (qit.eq.0.d0) then
              err=0.d0
            else
              err=d/dg
            endif
            dg=dg*1.25d0*omegam/omegak/sqrt(omegal)
            if (verbose.ge.2) print *,'method M Om_l > 0,',
     &        i,' terms; dg =',dg,' relerr =',err
c........recurse Beta functions from last 2 terms downward
          else
            qit=omm/(1.d0-omm)
            it=1+log(prec)/log(abs(qit))
            if (it.le.1) it=2
            if (verbose.ge.3) print *,'Om_m/(Om_k+Om_l) =',qit,
     &        '; use',it,' terms'
            if (.not.(it.le.itmax)) then
              it=itmax
              if (verbose.ge.3) print *,'will do only',it,' terms'
            endif
            if (verbose.ge.3) print *,
     &        'recurse from',it,'''th and',it-1,'''th terms downward'
            do ib=1,min(2,it)
              i=it-ib
              a(ib)=1.d0+1.5d0*i
              b(ib)=.5d0-.5d0*i
              xy(ib)=x*sqrt(y)*(x*sqrt(x/y))**i
              bet(ib)=betafp(x,y,a(ib),b(ib))
c        deal with NaN or Infinity
              if ((.not.(bet(ib).le.0.d0.or.bet(ib).ge.0.d0))
     &          .or.abs(bet(ib)).gt.dflmax()) then
                if (verbose.ge.3) print *,
     &            i,' starting B(',a(ib),',',b(ib),') =',bet(ib)
                bet(ib)=0.d0
              endif
            enddo
            dg=0.d0
            err=bet(1)
            do i=it,1,-1
              ib=1+mod(it-i,2)
              d=-dg*z*(dble(i)+HALF)/dble(i)
              dg=d+bet(ib)
              if (verbose.ge.3) then
                if (i.le.10.or.i.gt.it-10) then
                  print *,i,' err =',err,' dg =',dg,
     &              ' B(',a(ib),',',b(ib),') =',bet(ib),
     &              ' =',betafp(x,y,a(ib),b(ib))
                elseif (i.eq.11) then
                  print *,'...'
                endif
              endif
              if (i.eq.it.and.err.eq.0.d0) then
                err=bet(ib)
              else
                err=-err*z*(dble(i)+HALF)/dble(i)
              endif
c        recurse B_x(a,b) -> B_x(a-1,b)
              a(ib)=a(ib)-1.d0
              xy(ib)=xy(ib)/x
              bet(ib)=(bet(ib)*(a(ib)+b(ib))+xy(ib))/a(ib)
c        recurse B_x(a,b) -> B_x(a-1,b+1)
              a(ib)=a(ib)-1.d0
              xy(ib)=xy(ib)/x
              bet(ib)=(bet(ib)*b(ib)+xy(ib))/a(ib)
              b(ib)=b(ib)+1.d0
              xy(ib)=xy(ib)*y
c        recurse B_x(a,b) -> B_x(a-1,b)
              a(ib)=a(ib)-1.d0
              xy(ib)=xy(ib)/x
              bet(ib)=(bet(ib)*(a(ib)+b(ib))+xy(ib))/a(ib)
            enddo
            if (qit.eq.0.d0) then
              err=0.d0
            else
              err=err/dg
            endif
            dg=dg*1.25d0*omegam/omegak/sqrt(omegal)
            if (verbose.ge.2) print *,'method M Om_l > 0,',
     &        it,' terms; dg =',dg,' relerr =',err
          endif
          if (j.eq.2) dg=-dg
          g=g+dg
          if (verbose.ge.3) print *,'cumulative g =',g
        enddo
c--------method M (small Omega_m), Omega_k > 0, Omega_l < 0
      elseif (method/M.eq.1.and.omegal.lt.0.d0) then
        do j=1,2
          if (verbose.ge.3) then
            print *,'--------------------'
            print *,'method M, Omega_l < 0, part',j
          endif
          if (j.eq.2) then
            omm=omegam
            omk=omegak
            oml=omegal
          endif
          x=-oml/omk
          y=(1.d0-omm)/omk
          z=omm*sqrt(-oml/omk**3)
          zz=omm**2*(-oml)/omk**3
          if (verbose.ge.3) print *,'x, y, z, zz =',x,y,z,zz
          if (x.lt.0.d0.or.x.gt.1.d0) goto 340
c........recurse Beta functions from 1st 2 terms upward
          if (x.ge.0.d0.or.z.eq.0.d0) then
            if (z.eq.0.d0) then
              qit=0.d0
              it=1
            else
              qit=omm/(1.d0-omm)
              it=1+log(prec)/log(abs(qit))
              if (it.le.1) it=2
            endif
            if (verbose.ge.3) print *,'Om_m/(Om_k+Om_l) =',qit,
     &        '; expect',it,' terms'
            if (.not.(it.le.itmax)) then
              it=itmax
              if (verbose.ge.3) print *,'will do only',it,' terms'
            endif
            if (verbose.ge.3) print *,'recurse from 1st 2 terms upward'
            a(1)=.5d0
            b(1)=-.5d0
            a(2)=0.d0
            b(2)=-1.5d0
            xy(1)=sqrt(x/y)
            xy(2)=y**b(2)*z
            bet(1)=betafp(x,y,a(1),b(1))
            bet(2)=betafp(x,y,a(2),b(2))*z
            err=prec*abs(bet(1))
            q=1.d0
            dg=0.d0
            do i=1,it
              ib=1+mod(i-1,2)
              d=bet(ib)*q
              dg=dg+d
              if (verbose.ge.3) then
                if (i.le.10.or.i.gt.it-10.or.abs(d).le.err) then
                  print *,i,' d =',d,
     &              ' B(',a(ib),',',b(ib),') z^(i-1) =',bet(ib),
     &              ' =',betafp(x,y,a(ib),b(ib))*z**(i-1)
                elseif (i.eq.11) then
                  print *,'...'
                endif
              endif
              if (abs(d).le.err.or.i.eq.it) goto 160
              q=-q*(dble(i)+HALF)/dble(i)
c        recurse B_x(a,b) -> B_x(a,b-1)
              b(ib)=b(ib)-1.d0
              xy(ib)=xy(ib)/y
              bet(ib)=(bet(ib)*(a(ib)+b(ib))-xy(ib))/b(ib)
c        recurse B_x(a,b) -> B_x(a-1,b)
              a(ib)=a(ib)-1.d0
              xy(ib)=xy(ib)/x
              bet(ib)=(bet(ib)*(a(ib)+b(ib))+xy(ib))/a(ib)
c        recurse B_x(a,b) -> B_x(a,b-1)
              b(ib)=b(ib)-1.d0
              xy(ib)=xy(ib)/y
              bet(ib)=(bet(ib)*(a(ib)+b(ib))-xy(ib))/b(ib)
c        incorporate z factor into B, to avoid large/small numbers
              bet(ib)=bet(ib)*zz
              xy(ib)=xy(ib)*zz
            enddo
  160       if (qit.eq.0.d0) then
              err=0.d0
            else
              err=d/dg
            endif
            dg=dg*1.25d0*omegam/omegak/sqrt(-omegal)
            if (verbose.ge.2) print *,'method M Om_l < 0,',
     &        i,' terms; dg =',dg,' relerr =',err
c........recurse Beta functions from last 2 terms downward
          else
            qit=omm/(1.d0-omm)
            it=1+log(prec)/log(abs(qit))
            if (it.le.1) it=2
            if (verbose.ge.3) print *,'Om_m/(Om_k+Om_l) =',qit,
     &        '; use',it,' terms'
            if (.not.(it.le.itmax)) then
              it=itmax
              if (verbose.ge.3) print *,'will do only',it,' terms'
            endif
            if (verbose.ge.3) print *,
     &        'recurse from',it,'''th and',it-1,'''th terms downward'
            do ib=1,min(2,it)
              i=it-ib
              a(ib)=.5d0-.5d0*i
              b(ib)=-.5d0-1.d0*i
              xy(ib)=sqrt(x/y)*(z/(sqrt(x)*y))**i
              bet(ib)=betafp(x,y,a(ib),b(ib))*z**i
c        deal with NaN or Infinity
              if ((.not.(bet(ib).le.0.d0.or.bet(ib).ge.0.d0))
     &          .or.abs(bet(ib)).gt.dflmax()) then
                if (verbose.ge.3) print *,
     &            i,' starting B(',a(ib),',',b(ib),') z^i =',bet(ib)
                bet(ib)=0.d0
              endif
            enddo
            dg=0.d0
            err=bet(1)
            do i=it,1,-1
              ib=1+mod(it-i,2)
              d=-dg*(dble(i)+HALF)/dble(i)
              dg=d+bet(ib)
              if (verbose.ge.3) then
                if (i.le.10.or.i.gt.it-10) then
                  print *,i,' err =',err,' dg =',dg,
     &              ' B(',a(ib),',',b(ib),') z^(i-1) =',bet(ib),
     &              ' =',betafp(x,y,a(ib),b(ib))*z**(i-1)
                elseif (i.eq.11) then
                  print *,'...'
                endif
              endif
              if (i.eq.it.and.err.eq.0.d0) then
                err=bet(ib)
              else
                err=-err*(dble(i)+HALF)/dble(i)
              endif
c        incorporate z factor into B, to avoid large/small numbers
              bet(ib)=bet(ib)/zz
              xy(ib)=xy(ib)/zz
c        recurse B_x(a,b) -> B_x(a,b+1)
              bet(ib)=(bet(ib)*b(ib)+xy(ib))/(a(ib)+b(ib))
              b(ib)=b(ib)+1.d0
              xy(ib)=xy(ib)*y
c        recurse B_x(a,b) -> B_x(a+1,b)
              bet(ib)=(bet(ib)*a(ib)-xy(ib))/(a(ib)+b(ib))
              a(ib)=a(ib)+1.d0
              xy(ib)=xy(ib)*x
c        recurse B_x(a,b) -> B_x(a,b+1)
              bet(ib)=(bet(ib)*b(ib)+xy(ib))/(a(ib)+b(ib))
              b(ib)=b(ib)+1.d0
              xy(ib)=xy(ib)*y
            enddo
            if (qit.eq.0.d0) then
              err=0.d0
            else
              err=err/dg
            endif
            dg=dg*1.25d0*omegam/omegak/sqrt(-omegal)
            if (verbose.ge.2) print *,'method M Om_l < 0,',
     &        it,' terms; dg =',dg,' relerr =',err
          endif
          if (j.eq.1) dg=-dg
          g=g+dg
          if (verbose.ge.3) print *,'cumulative g =',g
        enddo
      endif
c--------dlnD/dlna
      f=-omegam/2.d0+omegal-1.d0+2.5d0*omegam/g
      if (verbose.ge.3) print *,'===================='
  200 ok=.true.
      return
c
c--------error returns
  320 if (verbose.ge.1)
     &  print *,'growl: Omega_m =',omegam,' should be >= 0'
      return
  330 if (verbose.ge.1)
     &  print *,'growl: Omega_k == 1-Omega_m-Omega_l =',omegak,
     &  ' should be > min[-3 Omega_m/2, -(27 Omega_m^2 Omega_l/4)^(1/3)]
     & =',min(-1.5d0*omegam,-(6.75d0*omegam**2*omegal)**THIRD)
      return
  340 if (verbose.ge.1) print *,'Beta function argument',x,
     &  ' should be in [0,1]'
      return
      end
c
c-----------------------------------------------------------------------
      subroutine growLLPR(omegam,omegal,gLLPR,fLLPR)
      real*8 omegam,omegal,gLLPR,fLLPR
c
c        parameters
      real*8 OMEXP
c        exponent preferred by Lahav et al
c Works better at low Omega_m with Omega_l = 0
c     parameter (OMEXP=0.6d0)
c        exponent preferred by Carroll et al
c Works better at low Omega_m in flat Universe
      parameter (OMEXP=4.d0/7.d0)
c *
c * Approximation to growth rate from
c * Lahav O., Lilje P. B., Primack J. R., Rees M., 1991, MNRAS, 251, 136
c * with consequent growth factor
c * Carroll S. M., Press W. H., Turner E. L., 1992, ARAA, 30, 499.
c *
      fLLPR=omegam**OMEXP+(1+omegam/2.d0)*omegal/70.d0
      gLLPR=2.5d0*omegam/(fLLPR+1.d0+omegam/2.d0-omegal)
c
c Following is dln(D_LLPR)/dlna
c Empirically, does not work as well as simple fLLPR
c     fLLPR=(3.d0/2.d0*omegam+omp*(4.d0+3.d0*omegam-6.d0*omegal)/7.d0
c    *  -omegal*(4.d0+omegam*(1.d0+omegam-2.*omegal))/140.d0)/den
c
      return
      end
c
